'use client'
import useShowToast from '@/app/hooks/useShowToast';
import { createCaseRequest, editCaseRequest } from '@/app/lib/cases/action';
import { InputWrapper, SelectWrapper } from '@/components/fields/Fields';
import FormComponent from '@/components/form/Form';
import prepareValidationSchema from '@/utils/validation-schema';
import { Box, Button, FormControl, FormLabel, Text, useColorModeValue } from "@chakra-ui/react";
import { useRouter } from 'next/navigation';
import { useRef } from 'react';
import { CaseInterface } from '../type/case';
import { formFields } from '../variables/formFields';
import { IbcInterface } from '@/types/settings/ibc/ibc';


interface formProps {
    isEdit: CaseInterface,
    ibcList: IbcInterface[];
    cycleDaysList: any;
}

const CaseForm = ({ isEdit, ibcList, cycleDaysList }: formProps) => {
    const formRef = useRef(null);
    const router = useRouter();
    const showToast = useShowToast();
    const textColor = useColorModeValue("navy.700", "white");
    const brandStars = useColorModeValue("brand.500", "brand.400");

    const formDetails = prepareValidationSchema(formFields);
    const { initialFormData, validationSchema } = formDetails;

    // Set initial values based on whether we are adding or editing
    const initialValues = isEdit ? { ...isEdit } : initialFormData;

    const handleSubmitClick = () => {
        formRef?.current?.externalHandleSubmit();
    };


    const handleCreate = async (values: FormData) => {
        try {
            const response = await createCaseRequest(values);
            showToast("Case Created!", response.message, "success");
            router.push("/admin/cases");
        } catch (error: any) {
            showToast("Failed to create", error.message, "error");
        }
    };

    const handleUpdate = async (values: FormData) => {
        try {
            const payload = { userId: isEdit.id, data: values };
            const response = await editCaseRequest(payload);
            showToast("Case Updated!", response.message, "success");
            router.push("/admin/cases");
        } catch (error: any) {
            showToast("Failed to create", error.message, "error");
        }
    };

    const sanitizeFormData = (values: FormData) => {
        // List the allowed fields that should be part of the form data
        const allowedFields = [
            "contract",
            "contract_account",
            "consumer_no",
            "meter_no",
            "address",
            "mru",
            "name",
            "rate_tariff",
            "due_bcm",
            "total_due_units",
            "total_due_billing",
            "last_payment_date",
            "last_payment_amount",
            "dues",
            "case_type",
            "ibc_id",
            "cycle_day_id"
        ];

        // Filter out any fields that are not in the allowed list

        const sanitizedData = values && Object.keys(values).reduce((acc: any, key: string) => {
            if (allowedFields.includes(key)) {
                acc[key] = values[key];
            }
            return acc;
        }, {});

        return sanitizedData;
    };



    const handleSubmit = (values: FormData) => {

        if (isEdit) {
            const sanitizedValues = sanitizeFormData(values);
            handleUpdate(sanitizedValues);
        } else {
            handleCreate(values);
        }
    };

    return (
        <>
            <FormComponent
                ref={formRef}
                validationSchema={validationSchema}
                initialValues={initialValues}
                // enableReinitialize // Ensures the form reinitializes with updated initial values
                submitForm={handleSubmit}

            >

                <Box display="flex" gap={15}>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Contract &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="contract"
                            defaultValue={isEdit?.contract}
                            id="contract"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="number"
                            fontWeight="500"
                        />
                    </FormControl>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Contract Account &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="contract_account"
                            defaultValue={isEdit?.contract_account}
                            id="contract_account"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="number"
                            fontWeight="500"
                        />
                    </FormControl>
                </Box>
                <Box display="flex" gap={15}>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Consumer No &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="consumer_no"
                            defaultValue={isEdit?.consumer_no}
                            id="consumer_no"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="text"
                            fontWeight="500"
                        />
                    </FormControl>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Meter No &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="meter_no"
                            defaultValue={isEdit?.meter_no}
                            id="meter_no"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="text"
                            fontWeight="500"
                        />
                    </FormControl>
                </Box>
                <Box display="flex" gap={15}>
                    <FormControl w="100%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Address &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="address"
                            defaultValue={isEdit?.address}
                            id="address"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="text"
                            fontWeight="500"
                        />
                    </FormControl>
                </Box>
                <Box display="flex" gap={15}>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            MRU &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="mru"
                            defaultValue={isEdit?.mru}
                            id="mru"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="text"
                            fontWeight="500"
                        />
                    </FormControl>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Name &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="name"
                            defaultValue={isEdit?.name}
                            id="name"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="text"
                            fontWeight="500"
                        />
                    </FormControl>
                </Box>
                <Box display="flex" gap={15}>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Rate Tariff &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="rate_tariff"
                            defaultValue={isEdit?.rate_tariff}
                            id="rate_tariff"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="text"
                            fontWeight="500"
                        />
                    </FormControl>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Due Bcm &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="due_bcm"
                            defaultValue={isEdit?.due_bcm}
                            id="due_bcm"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="text"
                            fontWeight="500"
                        />
                    </FormControl>
                </Box>
                <Box display="flex" gap={15}>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Total Due Units &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="total_due_units"
                            defaultValue={isEdit?.total_due_units}
                            id="total_due_units"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="number"
                            fontWeight="500"
                        />
                    </FormControl>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Total Due Billing &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="total_due_billing"
                            defaultValue={isEdit?.total_due_billing}
                            id="total_due_billing"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="number"
                            fontWeight="500"
                        />
                    </FormControl>
                </Box>
                <Box display="flex" gap={15}>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Last Payment Date &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="last_payment_date"
                            defaultValue={isEdit?.last_payment_date}
                            id="last_payment_date"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="date"
                            fontWeight="500"
                        />
                    </FormControl>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Last Payment Amount &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="last_payment_amount"
                            defaultValue={isEdit?.last_payment_amount}
                            id="last_payment_amount"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="number"
                            fontWeight="500"
                        />
                    </FormControl>
                </Box>
                <Box display="flex" gap={15}>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Dues &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="dues"
                            defaultValue={isEdit?.dues}
                            id="dues"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="number"
                            fontWeight="500"
                        />
                    </FormControl>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Case Type &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <InputWrapper
                            form
                            fieldName="case_type"
                            defaultValue={isEdit?.case_type}
                            id="case_type"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            type="number"
                            fontWeight="500"
                        />
                    </FormControl>
                </Box>

                <Box display="flex" gap={15}>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Select Ibc &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <SelectWrapper
                            form
                            fieldName="ibc_id"
                            defaultValue={isEdit?.ibc_id}
                            id="ibc_id"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            fontWeight="500"
                            options={ibcList?.map((item: IbcInterface) => ({ label: item.area_name, value: item.id }))}
                        />
                    </FormControl>
                    <FormControl w="50%">
                        <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                            Select Cycle Days &nbsp;<Text display="inline-block" color={brandStars}>*</Text>
                        </FormLabel>
                        <SelectWrapper
                            form
                            fieldName="cycle_day_id"
                            defaultValue={isEdit?.cycle_day_id}
                            id="cycle_day_id"
                            variant="auth"
                            w="full"
                            mb="2px"
                            fontSize="sm"
                            fontWeight="500"
                            options={cycleDaysList?.map((item: any) => ({ label: item.cycle_day, value: item.id }))}
                        />
                    </FormControl>
                </Box>


                <Button variant="brand" onClick={handleSubmitClick} mr={3}>
                    {isEdit ? "Save" : "Create"}
                </Button>

            </FormComponent>

        </>
    )
}

export default CaseForm